# coding=utf-8
from .constants import *
from .command import ApplicationCommand
from .error import ApplicationError
from .file import ApplicationFile
from .log import ApplicationLog
from .status import ApplicationStatus



# coding=utf-8
import threading

from machinetalk.protobuf.message_pb2 import Container
from machinetalk.protobuf.status_pb2 import (
    EMC_TASK_MODE_AUTO,
    EMC_TASK_MODE_MDI,
    EMC_TASK_INTERP_IDLE,
)
from ..common import ComponentBase, MessageObject, recurse_descriptor, recurse_message
from ..dns_sd import ServiceContainer, Service
from ..machinetalk_core.application.statusbase import StatusBase


class ApplicationStatus(ComponentBase, StatusBase, ServiceContainer):
    def __init__(self, debug=False):
        StatusBase.__init__(self, debuglevel=int(debug))
        ComponentBase.__init__(self)
        ServiceContainer.__init__(self)
        self.config_condition = threading.Condition(threading.Lock())
        self.io_condition = threading.Condition(threading.Lock())
        self.motion_condition = threading.Condition(threading.Lock())
        self.task_condition = threading.Condition(threading.Lock())
        self.interp_condition = threading.Condition(threading.Lock())
        self.synced_condition = threading.Condition(threading.Lock())
        self.debug = debug

        # callbacks
        self.on_synced_changed = []

        self.synced = False

        # status containers, also used to expose data
        self._io_data = None
        self._motion_data = None
        self._config_data = None
        self._task_data = None
        self._interp_data = None
        # required for object initialization
        self._container = Container()
        self._initialize_object('io')
        self._initialize_object('config')
        self._initialize_object('motion')
        self._initialize_object('task')
        self._initialize_object('interp')

        self._synced_channels = set()
        self.channels = {'motion', 'config', 'task', 'io', 'interp'}

        self._status_service = Service(type_='status')
        self.add_service(self._status_service)
        self.on_services_ready_changed.append(self._on_services_ready_changed)

    def _on_services_ready_changed(self, ready):
        self.status_uri = self._status_service.uri
        self.ready = ready

    # make sure locks are used when accessing properties
    # should we return a copy instead of the reference?
    @property
    def io(self):
        with self.io_condition:
            return self._io_data

    @property
    def config(self):
        with self.config_condition:
            return self._config_data

    @property
    def motion(self):
        with self.motion_condition:
            return self._motion_data

    @property
    def task(self):
        with self.task_condition:
            return self._task_data

    @property
    def interp(self):
        with self.interp_condition:
            return self._interp_data

    def wait_synced(self, timeout=None):
        with self.synced_condition:
            if self.synced:
                return True
            self.synced_condition.wait(timeout=timeout)
            return self.synced

    def wait_config_updated(self, timeout=None):
        with self.config_condition:
            self.config_condition.wait(timeout=timeout)

    def wait_io_updated(self, timeout=None):
        with self.io_condition:
            self.io_condition.wait(timeout=timeout)

    def wait_motion_updated(self, timeout=None):
        with self.motion_condition:
            self.motion_condition.wait(timeout=timeout)

    def wait_task_updated(self, timeout=None):
        with self.task_condition:
            self.task_condition.wait(timeout=timeout)

    def wait_interp_updated(self, timeout=None):
        with self.interp_condition:
            self.interp_condition.wait(timeout=timeout)

    def emcstat_full_update_received(self, topic, rx):
        self._emcstat_update_received(topic, rx)
        self._update_synced_channels(topic)

    def emcstat_incremental_update_received(self, topic, rx):
        self._emcstat_update_received(topic, rx)

    def _emcstat_update_received(self, topic, rx):
        topic = six.ensure_str(topic)
        if topic == 'motion' and rx.HasField('emc_status_motion'):
            self._update_motion_object(rx.emc_status_motion)
        elif topic == 'config' and rx.HasField('emc_status_config'):
            self._update_config_object(rx.emc_status_config)
        elif topic == 'io' and rx.HasField('emc_status_io'):
            self._update_io_object(rx.emc_status_io)
        elif topic == 'task' and rx.HasField('emc_status_task'):
            self._update_task_object(rx.emc_status_task)
        elif topic == 'interp' and rx.HasField('emc_status_interp'):
            self._update_interp_object(rx.emc_status_interp)

    def _update_synced_channels(self, channel):
        self._synced_channels.add(channel)
        if (self._synced_channels == self.channels) and not self.synced:
            self.channels_synced()

    # slot
    def sync_status(self):
        self._update_synced(True)

    # slot
    def unsync_status(self):
        self._synced_channels.clear()
        self._update_synced(False)

    def _update_synced(self, synced):
        with self.synced_condition:
            self.synced = synced
            self.synced_condition.notify()
        for cb in self.on_synced_changed:
            cb(synced)

    # slot
    def update_topics(self):
        self.clear_status_topics()
        for channel in self.channels:
            self.add_status_topic(channel)
            self._initialize_object(channel)

    def _initialize_object(self, channel):
        if channel == 'io':
            self._io_data = MessageObject()
            recurse_descriptor(self._container.emc_status_io.DESCRIPTOR, self._io_data)
        elif channel == 'config':
            self._config_data = MessageObject()
            recurse_descriptor(
                self._container.emc_status_config.DESCRIPTOR, self._config_data
            )
        elif channel == 'motion':
            self._motion_data = MessageObject()
            recurse_descriptor(
                self._container.emc_status_motion.DESCRIPTOR, self._motion_data
            )
        elif channel == 'task':
            self._task_data = MessageObject()
            recurse_descriptor(
                self._container.emc_status_task.DESCRIPTOR, self._task_data
            )
        elif channel == 'interp':
            self._interp_data = MessageObject()
            recurse_descriptor(
                self._container.emc_status_interp.DESCRIPTOR, self._interp_data
            )

    def _update_motion_object(self, data):
        with self.motion_condition:
            recurse_message(data, self._motion_data)
            self.motion_condition.notify()

    def _update_config_object(self, data):
        with self.config_condition:
            recurse_message(data, self._config_data)
            self.config_condition.notify()

    def _update_io_object(self, data):
        with self.io_condition:
            recurse_message(data, self._io_data)
            self.io_condition.notify()

    def _update_task_object(self, data):
        with self.task_condition:
            recurse_message(data, self._task_data)
            self._update_running()
            self.task_condition.notify()

    def _update_interp_object(self, data):
        with self.interp_condition:
            recurse_message(data, self._interp_data)
            self._update_running()
            self.interp_condition.notify()

    def _update_running(self):
        running = (
            self._task_data.task_mode == EMC_TASK_MODE_AUTO
            or self._task_data.task_mode == EMC_TASK_MODE_MDI
        ) and self._interp_data.interp_state == EMC_TASK_INTERP_IDLE

        self.running = running



# coding=utf-8
from fysom import Fysom
from ..common.rpcclient import RpcClient

import machinetalk.protobuf.types_pb2 as pb
from machinetalk.protobuf.message_pb2 import Container


class ConfigBase(object):
    def __init__(self, debuglevel=0, debugname='Config Base'):
        self.debuglevel = debuglevel
        self.debugname = debugname
        self._error_string = ''
        self.on_error_string_changed = []

        # Config
        self._config_channel = RpcClient(debuglevel=debuglevel)
        self._config_channel.debugname = '%s - %s' % (self.debugname, 'config')
        self._config_channel.on_state_changed.append(self._config_channel_state_changed)
        self._config_channel.on_socket_message_received.append(
            self._config_channel_message_received
        )
        # more efficient to reuse protobuf messages
        self._config_rx = Container()
        self._config_tx = Container()

        # callbacks
        self.on_config_message_received = []
        self.on_state_changed = []

        # fsm
        self._fsm = Fysom(
            {
                'initial': 'down',
                'events': [
                    {'name': 'connect', 'src': 'down', 'dst': 'trying'},
                    {'name': 'config_up', 'src': 'trying', 'dst': 'listing'},
                    {'name': 'disconnect', 'src': 'trying', 'dst': 'down'},
                    {'name': 'application_retrieved', 'src': 'listing', 'dst': 'up'},
                    {'name': 'config_trying', 'src': 'listing', 'dst': 'trying'},
                    {'name': 'disconnect', 'src': 'listing', 'dst': 'down'},
                    {'name': 'config_trying', 'src': 'up', 'dst': 'trying'},
                    {'name': 'load_application', 'src': 'up', 'dst': 'loading'},
                    {'name': 'disconnect', 'src': 'up', 'dst': 'down'},
                    {'name': 'application_loaded', 'src': 'loading', 'dst': 'up'},
                    {'name': 'config_trying', 'src': 'loading', 'dst': 'trying'},
                    {'name': 'disconnect', 'src': 'loading', 'dst': 'down'},
                ],
            }
        )

        self._fsm.ondown = self._on_fsm_down
        self._fsm.onafterconnect = self._on_fsm_connect
        self._fsm.ontrying = self._on_fsm_trying
        self._fsm.onafterconfig_up = self._on_fsm_config_up
        self._fsm.onafterdisconnect = self._on_fsm_disconnect
        self._fsm.onlisting = self._on_fsm_listing
        self._fsm.onafterapplication_retrieved = self._on_fsm_application_retrieved
        self._fsm.onafterconfig_trying = self._on_fsm_config_trying
        self._fsm.onup = self._on_fsm_up
        self._fsm.onafterload_application = self._on_fsm_load_application
        self._fsm.onleaveup = self._on_fsm_up_exit
        self._fsm.onloading = self._on_fsm_loading
        self._fsm.onafterapplication_loaded = self._on_fsm_application_loaded

    def _on_fsm_down(self, _):
        if self.debuglevel > 0:
            print('[%s]: state DOWN' % self.debugname)
        for cb in self.on_state_changed:
            cb('down')
        return True

    def _on_fsm_connect(self, _):
        if self.debuglevel > 0:
            print('[%s]: event CONNECT' % self.debugname)
        self.start_config_channel()
        return True

    def _on_fsm_trying(self, _):
        if self.debuglevel > 0:
            print('[%s]: state TRYING' % self.debugname)
        for cb in self.on_state_changed:
            cb('trying')
        return True

    def _on_fsm_config_up(self, _):
        if self.debuglevel > 0:
            print('[%s]: event CONFIG UP' % self.debugname)
        self.send_list_applications()
        return True

    def _on_fsm_disconnect(self, _):
        if self.debuglevel > 0:
            print('[%s]: event DISCONNECT' % self.debugname)
        self.stop_config_channel()
        return True

    def _on_fsm_listing(self, _):
        if self.debuglevel > 0:
            print('[%s]: state LISTING' % self.debugname)
        for cb in self.on_state_changed:
            cb('listing')
        return True

    def _on_fsm_application_retrieved(self, _):
        if self.debuglevel > 0:
            print('[%s]: event APPLICATION RETRIEVED' % self.debugname)
        return True

    def _on_fsm_config_trying(self, _):
        if self.debuglevel > 0:
            print('[%s]: event CONFIG TRYING' % self.debugname)
        return True

    def _on_fsm_up(self, _):
        if self.debuglevel > 0:
            print('[%s]: state UP entry' % self.debugname)
        self.sync_config()
        if self.debuglevel > 0:
            print('[%s]: state UP' % self.debugname)
        for cb in self.on_state_changed:
            cb('up')
        return True

    def _on_fsm_load_application(self, _):
        if self.debuglevel > 0:
            print('[%s]: event LOAD APPLICATION' % self.debugname)
        return True

    def _on_fsm_up_exit(self, _):
        if self.debuglevel > 0:
            print('[%s]: state UP exit' % self.debugname)
        self.unsync_config()
        return True

    def _on_fsm_loading(self, _):
        if self.debuglevel > 0:
            print('[%s]: state LOADING' % self.debugname)
        for cb in self.on_state_changed:
            cb('loading')
        return True

    def _on_fsm_application_loaded(self, _):
        if self.debuglevel > 0:
            print('[%s]: event APPLICATION LOADED' % self.debugname)
        return True

    @property
    def error_string(self):
        return self._error_string

    @error_string.setter
    def error_string(self, string):
        if self._error_string is string:
            return
        self._error_string = string
        for cb in self.on_error_string_changed:
            cb(string)

    @property
    def config_uri(self):
        return self._config_channel.socket_uri

    @config_uri.setter
    def config_uri(self, value):
        self._config_channel.socket_uri = value

    def sync_config(self):
        print('WARNING: slot sync config unimplemented')

    def unsync_config(self):
        print('WARNING: slot unsync config unimplemented')

    def start(self):
        if self._fsm.isstate('down'):
            self._fsm.connect()

    def stop(self):
        if self._fsm.isstate('trying'):
            self._fsm.disconnect()
        elif self._fsm.isstate('listing'):
            self._fsm.disconnect()
        elif self._fsm.isstate('up'):
            self._fsm.disconnect()
        elif self._fsm.isstate('loading'):
            self._fsm.disconnect()

    def start_config_channel(self):
        self._config_channel.start()

    def stop_config_channel(self):
        self._config_channel.stop()

    # process all messages received on config
    def _config_channel_message_received(self, rx):

        # react to describe application message
        if rx.type == pb.MT_DESCRIBE_APPLICATION:
            if self._fsm.isstate('listing'):
                self._fsm.application_retrieved()
            self.describe_application_received(rx)

        # react to application detail message
        elif rx.type == pb.MT_APPLICATION_DETAIL:
            if self._fsm.isstate('loading'):
                self._fsm.application_loaded()
            self.application_detail_received(rx)

        # react to error message
        elif rx.type == pb.MT_ERROR:
            # update error string with note
            self.error_string = ''
            for note in rx.note:
                self.error_string += note + '\n'

        for cb in self.on_config_message_received:
            cb(rx)

    def describe_application_received(self, rx):
        print('SLOT describe application unimplemented')

    def application_detail_received(self, rx):
        print('SLOT application detail unimplemented')

    def send_config_message(self, msg_type, tx):
        self._config_channel.send_socket_message(msg_type, tx)

        if msg_type == pb.MT_RETRIEVE_APPLICATION:
            if self._fsm.isstate('up'):
                self._fsm.load_application()

    def send_list_applications(self):
        tx = self._config_tx
        self.send_config_message(pb.MT_LIST_APPLICATIONS, tx)

    def send_retrieve_application(self, tx):
        self.send_config_message(pb.MT_RETRIEVE_APPLICATION, tx)

    def _config_channel_state_changed(self, state):

        if state == 'trying':
            if self._fsm.isstate('listing'):
                self._fsm.config_trying()
            elif self._fsm.isstate('up'):
                self._fsm.config_trying()
            elif self._fsm.isstate('loading'):
                self._fsm.config_trying()

        elif state == 'up':
            if self._fsm.isstate('trying'):
                self._fsm.config_up()



#!/usr/bin/env python

import time
import sys
import threading

from pymachinetalk.dns_sd import ServiceDiscovery
import pymachinetalk.halremote as halremote


class BasicClass(threading.Thread):
    def __init__(self):
        threading.Thread.__init__(self)
        self.daemon = True

    def run(self):
        timeout = 2.0

        sd = ServiceDiscovery()
        halrcomp = halremote.RemoteComponent('anddemo')
        halrcomp.newpin('button0', halremote.HAL_BIT, halremote.HAL_OUT)
        halrcomp.newpin('button1', halremote.HAL_BIT, halremote.HAL_OUT)
        halrcomp.newpin('led', halremote.HAL_BIT, halremote.HAL_IN)
        sd.register(halrcomp)

        sd.start()
        print('waiting for component connected')
        assert halrcomp.wait_connected(timeout)
        print('component connected')
        print('stopping service discovery')
        sd.stop()
        print('completed')


def main():
    basic = BasicClass()
    basic.start()

    # wait for all threads to terminate
    while threading.active_count() > 1:
        time.sleep(0.1)

    print("threads stopped")
    sys.exit(0)


if __name__ == "__main__":
    main()



# coding=utf-8
from fysom import Fysom
from ..application.statussubscribe import StatusSubscribe

import machinetalk.protobuf.types_pb2 as pb
from machinetalk.protobuf.message_pb2 import Container


class StatusBase(object):
    def __init__(self, debuglevel=0, debugname='Status Base'):
        self.debuglevel = debuglevel
        self.debugname = debugname
        self._error_string = ''
        self.on_error_string_changed = []

        # Status
        self._status_channel = StatusSubscribe(debuglevel=debuglevel)
        self._status_channel.debugname = '%s - %s' % (self.debugname, 'status')
        self._status_channel.on_state_changed.append(self._status_channel_state_changed)
        self._status_channel.on_socket_message_received.append(
            self._status_channel_message_received
        )
        # more efficient to reuse protobuf messages
        self._status_rx = Container()

        # callbacks
        self.on_status_message_received = []
        self.on_state_changed = []

        # fsm
        self._fsm = Fysom(
            {
                'initial': 'down',
                'events': [
                    {'name': 'connect', 'src': 'down', 'dst': 'trying'},
                    {'name': 'status_up', 'src': 'trying', 'dst': 'syncing'},
                    {'name': 'disconnect', 'src': 'trying', 'dst': 'down'},
                    {'name': 'channels_synced', 'src': 'syncing', 'dst': 'up'},
                    {'name': 'status_trying', 'src': 'syncing', 'dst': 'trying'},
                    {'name': 'disconnect', 'src': 'syncing', 'dst': 'down'},
                    {'name': 'status_trying', 'src': 'up', 'dst': 'trying'},
                    {'name': 'disconnect', 'src': 'up', 'dst': 'down'},
                ],
            }
        )

        self._fsm.ondown = self._on_fsm_down
        self._fsm.onafterconnect = self._on_fsm_connect
        self._fsm.ontrying = self._on_fsm_trying
        self._fsm.onafterstatus_up = self._on_fsm_status_up
        self._fsm.onafterdisconnect = self._on_fsm_disconnect
        self._fsm.onsyncing = self._on_fsm_syncing
        self._fsm.onafterchannels_synced = self._on_fsm_channels_synced
        self._fsm.onafterstatus_trying = self._on_fsm_status_trying
        self._fsm.onup = self._on_fsm_up
        self._fsm.onleaveup = self._on_fsm_up_exit

    def _on_fsm_down(self, _):
        if self.debuglevel > 0:
            print('[%s]: state DOWN' % self.debugname)
        for cb in self.on_state_changed:
            cb('down')
        return True

    def _on_fsm_connect(self, _):
        if self.debuglevel > 0:
            print('[%s]: event CONNECT' % self.debugname)
        self.update_topics()
        self.start_status_channel()
        return True

    def _on_fsm_trying(self, _):
        if self.debuglevel > 0:
            print('[%s]: state TRYING' % self.debugname)
        for cb in self.on_state_changed:
            cb('trying')
        return True

    def _on_fsm_status_up(self, _):
        if self.debuglevel > 0:
            print('[%s]: event STATUS UP' % self.debugname)
        return True

    def _on_fsm_disconnect(self, _):
        if self.debuglevel > 0:
            print('[%s]: event DISCONNECT' % self.debugname)
        self.stop_status_channel()
        return True

    def _on_fsm_syncing(self, _):
        if self.debuglevel > 0:
            print('[%s]: state SYNCING' % self.debugname)
        for cb in self.on_state_changed:
            cb('syncing')
        return True

    def _on_fsm_channels_synced(self, _):
        if self.debuglevel > 0:
            print('[%s]: event CHANNELS SYNCED' % self.debugname)
        return True

    def _on_fsm_status_trying(self, _):
        if self.debuglevel > 0:
            print('[%s]: event STATUS TRYING' % self.debugname)
        return True

    def _on_fsm_up(self, _):
        if self.debuglevel > 0:
            print('[%s]: state UP entry' % self.debugname)
        self.sync_status()
        if self.debuglevel > 0:
            print('[%s]: state UP' % self.debugname)
        for cb in self.on_state_changed:
            cb('up')
        return True

    def _on_fsm_up_exit(self, _):
        if self.debuglevel > 0:
            print('[%s]: state UP exit' % self.debugname)
        self.unsync_status()
        return True

    @property
    def error_string(self):
        return self._error_string

    @error_string.setter
    def error_string(self, string):
        if self._error_string is string:
            return
        self._error_string = string
        for cb in self.on_error_string_changed:
            cb(string)

    @property
    def status_uri(self):
        return self._status_channel.socket_uri

    @status_uri.setter
    def status_uri(self, value):
        self._status_channel.socket_uri = value

    def sync_status(self):
        print('WARNING: slot sync status unimplemented')

    def unsync_status(self):
        print('WARNING: slot unsync status unimplemented')

    def update_topics(self):
        print('WARNING: slot update topics unimplemented')

    def start(self):
        if self._fsm.isstate('down'):
            self._fsm.connect()

    def stop(self):
        if self._fsm.isstate('trying'):
            self._fsm.disconnect()
        elif self._fsm.isstate('up'):
            self._fsm.disconnect()

    def channels_synced(self):
        if self._fsm.isstate('syncing'):
            self._fsm.channels_synced()

    def add_status_topic(self, name):
        self._status_channel.add_socket_topic(name)

    def remove_status_topic(self, name):
        self._status_channel.remove_socket_topic(name)

    def clear_status_topics(self):
        self._status_channel.clear_socket_topics()

    def start_status_channel(self):
        self._status_channel.start()

    def stop_status_channel(self):
        self._status_channel.stop()

    # process all messages received on status
    def _status_channel_message_received(self, identity, rx):

        # react to emcstat full update message
        if rx.type == pb.MT_EMCSTAT_FULL_UPDATE:
            self.emcstat_full_update_received(identity, rx)

        # react to emcstat incremental update message
        elif rx.type == pb.MT_EMCSTAT_INCREMENTAL_UPDATE:
            self.emcstat_incremental_update_received(identity, rx)

        for cb in self.on_status_message_received:
            cb(identity, rx)

    def emcstat_full_update_received(self, identity, rx):
        print('SLOT emcstat full update unimplemented')

    def emcstat_incremental_update_received(self, identity, rx):
        print('SLOT emcstat incremental update unimplemented')

    def _status_channel_state_changed(self, state):

        if state == 'trying':
            if self._fsm.isstate('up'):
                self._fsm.status_trying()

        elif state == 'trying':
            if self._fsm.isstate('syncing'):
                self._fsm.status_trying()

        elif state == 'up':
            if self._fsm.isstate('trying'):
                self._fsm.status_up()



# coding=utf-8
from fysom import Fysom
from ..common.rpcclient import RpcClient

import machinetalk.protobuf.types_pb2 as pb
from machinetalk.protobuf.message_pb2 import Container


class CommandBase(object):
    def __init__(self, debuglevel=0, debugname='Command Base'):
        self.debuglevel = debuglevel
        self.debugname = debugname
        self._error_string = ''
        self.on_error_string_changed = []

        # Command
        self._command_channel = RpcClient(debuglevel=debuglevel)
        self._command_channel.debugname = '%s - %s' % (self.debugname, 'command')
        self._command_channel.on_state_changed.append(
            self._command_channel_state_changed
        )
        self._command_channel.on_socket_message_received.append(
            self._command_channel_message_received
        )
        # more efficient to reuse protobuf messages
        self._command_rx = Container()
        self._command_tx = Container()

        # callbacks
        self.on_command_message_received = []
        self.on_state_changed = []

        # fsm
        self._fsm = Fysom(
            {
                'initial': 'down',
                'events': [
                    {'name': 'connect', 'src': 'down', 'dst': 'trying'},
                    {'name': 'command_up', 'src': 'trying', 'dst': 'up'},
                    {'name': 'disconnect', 'src': 'trying', 'dst': 'down'},
                    {'name': 'command_trying', 'src': 'up', 'dst': 'trying'},
                    {'name': 'disconnect', 'src': 'up', 'dst': 'down'},
                ],
            }
        )

        self._fsm.ondown = self._on_fsm_down
        self._fsm.onafterconnect = self._on_fsm_connect
        self._fsm.ontrying = self._on_fsm_trying
        self._fsm.onaftercommand_up = self._on_fsm_command_up
        self._fsm.onafterdisconnect = self._on_fsm_disconnect
        self._fsm.onup = self._on_fsm_up
        self._fsm.onaftercommand_trying = self._on_fsm_command_trying
        self._fsm.onleaveup = self._on_fsm_up_exit

    def _on_fsm_down(self, _):
        if self.debuglevel > 0:
            print('[%s]: state DOWN' % self.debugname)
        for cb in self.on_state_changed:
            cb('down')
        return True

    def _on_fsm_connect(self, _):
        if self.debuglevel > 0:
            print('[%s]: event CONNECT' % self.debugname)
        self.start_command_channel()
        return True

    def _on_fsm_trying(self, _):
        if self.debuglevel > 0:
            print('[%s]: state TRYING' % self.debugname)
        for cb in self.on_state_changed:
            cb('trying')
        return True

    def _on_fsm_command_up(self, _):
        if self.debuglevel > 0:
            print('[%s]: event COMMAND UP' % self.debugname)
        return True

    def _on_fsm_disconnect(self, _):
        if self.debuglevel > 0:
            print('[%s]: event DISCONNECT' % self.debugname)
        self.stop_command_channel()
        self.clear_connected()
        return True

    def _on_fsm_up(self, _):
        if self.debuglevel > 0:
            print('[%s]: state UP entry' % self.debugname)
        self.set_connected()
        if self.debuglevel > 0:
            print('[%s]: state UP' % self.debugname)
        for cb in self.on_state_changed:
            cb('up')
        return True

    def _on_fsm_command_trying(self, _):
        if self.debuglevel > 0:
            print('[%s]: event COMMAND TRYING' % self.debugname)
        return True

    def _on_fsm_up_exit(self, _):
        if self.debuglevel > 0:
            print('[%s]: state UP exit' % self.debugname)
        self.clear_connected()
        return True

    @property
    def error_string(self):
        return self._error_string

    @error_string.setter
    def error_string(self, string):
        if self._error_string is string:
            return
        self._error_string = string
        for cb in self.on_error_string_changed:
            cb(string)

    @property
    def command_uri(self):
        return self._command_channel.socket_uri

    @command_uri.setter
    def command_uri(self, value):
        self._command_channel.socket_uri = value

    def set_connected(self):
        print('WARNING: slot set connected unimplemented')

    def clear_connected(self):
        print('WARNING: slot clear connected unimplemented')

    def start(self):
        if self._fsm.isstate('down'):
            self._fsm.connect()

    def stop(self):
        if self._fsm.isstate('trying'):
            self._fsm.disconnect()
        elif self._fsm.isstate('up'):
            self._fsm.disconnect()

    def start_command_channel(self):
        self._command_channel.start()

    def stop_command_channel(self):
        self._command_channel.stop()

    # process all messages received on command
    def _command_channel_message_received(self, rx):

        # react to emccmd executed message
        if rx.type == pb.MT_EMCCMD_EXECUTED:
            self.emccmd_executed_received(rx)

        # react to emccmd completed message
        elif rx.type == pb.MT_EMCCMD_COMPLETED:
            self.emccmd_completed_received(rx)

        # react to error message
        elif rx.type == pb.MT_ERROR:
            # update error string with note
            self.error_string = ''
            for note in rx.note:
                self.error_string += note + '\n'

        for cb in self.on_command_message_received:
            cb(rx)

    def emccmd_executed_received(self, rx):
        print('SLOT emccmd executed unimplemented')

    def emccmd_completed_received(self, rx):
        print('SLOT emccmd completed unimplemented')

    def send_command_message(self, msg_type, tx):
        self._command_channel.send_socket_message(msg_type, tx)

    def send_emc_task_abort(self, tx):
        self.send_command_message(pb.MT_EMC_TASK_ABORT, tx)

    def send_emc_task_plan_run(self, tx):
        self.send_command_message(pb.MT_EMC_TASK_PLAN_RUN, tx)

    def send_emc_task_plan_pause(self, tx):
        self.send_command_message(pb.MT_EMC_TASK_PLAN_PAUSE, tx)

    def send_emc_task_plan_step(self, tx):
        self.send_command_message(pb.MT_EMC_TASK_PLAN_STEP, tx)

    def send_emc_task_plan_resume(self, tx):
        self.send_command_message(pb.MT_EMC_TASK_PLAN_RESUME, tx)

    def send_emc_set_debug(self, tx):
        self.send_command_message(pb.MT_EMC_SET_DEBUG, tx)

    def send_emc_coolant_flood_on(self, tx):
        self.send_command_message(pb.MT_EMC_COOLANT_FLOOD_ON, tx)

    def send_emc_coolant_flood_off(self, tx):
        self.send_command_message(pb.MT_EMC_COOLANT_FLOOD_OFF, tx)

    def send_emc_axis_home(self, tx):
        self.send_command_message(pb.MT_EMC_AXIS_HOME, tx)

    def send_emc_axis_jog(self, tx):
        self.send_command_message(pb.MT_EMC_AXIS_JOG, tx)

    def send_emc_axis_abort(self, tx):
        self.send_command_message(pb.MT_EMC_AXIS_ABORT, tx)

    def send_emc_axis_incr_jog(self, tx):
        self.send_command_message(pb.MT_EMC_AXIS_INCR_JOG, tx)

    def send_emc_tool_load_tool_table(self, tx):
        self.send_command_message(pb.MT_EMC_TOOL_LOAD_TOOL_TABLE, tx)

    def send_emc_tool_update_tool_table(self, tx):
        self.send_command_message(pb.MT_EMC_TOOL_UPDATE_TOOL_TABLE, tx)

    def send_emc_task_plan_execute(self, tx):
        self.send_command_message(pb.MT_EMC_TASK_PLAN_EXECUTE, tx)

    def send_emc_coolant_mist_on(self, tx):
        self.send_command_message(pb.MT_EMC_COOLANT_MIST_ON, tx)

    def send_emc_coolant_mist_off(self, tx):
        self.send_command_message(pb.MT_EMC_COOLANT_MIST_OFF, tx)

    def send_emc_task_plan_init(self, tx):
        self.send_command_message(pb.MT_EMC_TASK_PLAN_INIT, tx)

    def send_emc_task_plan_open(self, tx):
        self.send_command_message(pb.MT_EMC_TASK_PLAN_OPEN, tx)

    def send_emc_task_plan_set_optional_stop(self, tx):
        self.send_command_message(pb.MT_EMC_TASK_PLAN_SET_OPTIONAL_STOP, tx)

    def send_emc_task_plan_set_block_delete(self, tx):
        self.send_command_message(pb.MT_EMC_TASK_PLAN_SET_BLOCK_DELETE, tx)

    def send_emc_task_set_mode(self, tx):
        self.send_command_message(pb.MT_EMC_TASK_SET_MODE, tx)

    def send_emc_task_set_state(self, tx):
        self.send_command_message(pb.MT_EMC_TASK_SET_STATE, tx)

    def send_emc_traj_set_so_enable(self, tx):
        self.send_command_message(pb.MT_EMC_TRAJ_SET_SO_ENABLE, tx)

    def send_emc_traj_set_fh_enable(self, tx):
        self.send_command_message(pb.MT_EMC_TRAJ_SET_FH_ENABLE, tx)

    def send_emc_traj_set_fo_enable(self, tx):
        self.send_command_message(pb.MT_EMC_TRAJ_SET_FO_ENABLE, tx)

    def send_emc_traj_set_max_velocity(self, tx):
        self.send_command_message(pb.MT_EMC_TRAJ_SET_MAX_VELOCITY, tx)

    def send_emc_traj_set_mode(self, tx):
        self.send_command_message(pb.MT_EMC_TRAJ_SET_MODE, tx)

    def send_emc_traj_set_scale(self, tx):
        self.send_command_message(pb.MT_EMC_TRAJ_SET_SCALE, tx)

    def send_emc_traj_set_rapid_scale(self, tx):
        self.send_command_message(pb.MT_EMC_TRAJ_SET_RAPID_SCALE, tx)

    def send_emc_traj_set_spindle_scale(self, tx):
        self.send_command_message(pb.MT_EMC_TRAJ_SET_SPINDLE_SCALE, tx)

    def send_emc_traj_set_teleop_enable(self, tx):
        self.send_command_message(pb.MT_EMC_TRAJ_SET_TELEOP_ENABLE, tx)

    def send_emc_traj_set_teleop_vector(self, tx):
        self.send_command_message(pb.MT_EMC_TRAJ_SET_TELEOP_VECTOR, tx)

    def send_emc_tool_set_offset(self, tx):
        self.send_command_message(pb.MT_EMC_TOOL_SET_OFFSET, tx)

    def send_emc_axis_override_limits(self, tx):
        self.send_command_message(pb.MT_EMC_AXIS_OVERRIDE_LIMITS, tx)

    def send_emc_spindle_constant(self, tx):
        self.send_command_message(pb.MT_EMC_SPINDLE_CONSTANT, tx)

    def send_emc_spindle_decrease(self, tx):
        self.send_command_message(pb.MT_EMC_SPINDLE_DECREASE, tx)

    def send_emc_spindle_increase(self, tx):
        self.send_command_message(pb.MT_EMC_SPINDLE_INCREASE, tx)

    def send_emc_spindle_off(self, tx):
        self.send_command_message(pb.MT_EMC_SPINDLE_OFF, tx)

    def send_emc_spindle_on(self, tx):
        self.send_command_message(pb.MT_EMC_SPINDLE_ON, tx)

    def send_emc_spindle_brake_engage(self, tx):
        self.send_command_message(pb.MT_EMC_SPINDLE_BRAKE_ENGAGE, tx)

    def send_emc_spindle_brake_release(self, tx):
        self.send_command_message(pb.MT_EMC_SPINDLE_BRAKE_RELEASE, tx)

    def send_emc_motion_set_aout(self, tx):
        self.send_command_message(pb.MT_EMC_MOTION_SET_AOUT, tx)

    def send_emc_motion_set_dout(self, tx):
        self.send_command_message(pb.MT_EMC_MOTION_SET_DOUT, tx)

    def send_emc_motion_adaptive(self, tx):
        self.send_command_message(pb.MT_EMC_MOTION_ADAPTIVE, tx)

    def send_emc_axis_set_max_position_limit(self, tx):
        self.send_command_message(pb.MT_EMC_AXIS_SET_MAX_POSITION_LIMIT, tx)

    def send_emc_axis_set_min_position_limit(self, tx):
        self.send_command_message(pb.MT_EMC_AXIS_SET_MIN_POSITION_LIMIT, tx)

    def send_emc_axis_unhome(self, tx):
        self.send_command_message(pb.MT_EMC_AXIS_UNHOME, tx)

    def send_shutdown(self, tx):
        self.send_command_message(pb.MT_SHUTDOWN, tx)

    def _command_channel_state_changed(self, state):

        if state == 'trying':
            if self._fsm.isstate('up'):
                self._fsm.command_trying()

        elif state == 'up':
            if self._fsm.isstate('trying'):
                self._fsm.command_up()



#!/usr/bin/env python
import sys
import os
import time
import threading
import curses

from machinekit import config
from pymachinetalk.dns_sd import ServiceDiscovery, ServiceDiscoveryFilter
from pymachinetalk.application import ApplicationStatus
from pymachinetalk.application import ApplicationCommand
from pymachinetalk.application import ApplicationError
from pymachinetalk.application import ApplicationFile
import pymachinetalk.application as application
import pymachinetalk.halremote as halremote

if sys.version_info >= (3, 0):
    import configparser
else:
    import ConfigParser as configparser


class TerminalUI(object):
    def __init__(self, uuid, use_curses, debug=False):
        sd_filter = ServiceDiscoveryFilter(txt_records={'uuid': uuid})
        self.sd = ServiceDiscovery(filter_=sd_filter)

        halrcomp = halremote.component('test')
        halrcomp.debug = debug
        halrcomp.newpin("coolant-iocontrol", halremote.HAL_BIT, halremote.HAL_IN)
        halrcomp.newpin("coolant", halremote.HAL_BIT, halremote.HAL_OUT)
        self.halrcomp = halrcomp
        self.sd.register(halrcomp)

        halrcomp2 = halremote.RemoteComponent(name='test2', debug=debug)
        halrcomp2.newpin("coolant-iocontrol", halremote.HAL_BIT, halremote.HAL_IN)
        halrcomp2.newpin("coolant", halremote.HAL_BIT, halremote.HAL_OUT)
        self.halrcomp2 = halrcomp2
        self.sd.register(halrcomp2)

        self.status = ApplicationStatus(debug=debug)
        self.status.on_synced_changed.append(self._on_status_synced)
        self.sd.register(self.status)
        self.command = ApplicationCommand(debug=debug)
        self.sd.register(self.command)
        self.error = ApplicationError(debug=debug)
        self.sd.register(self.error)
        self.fileservice = ApplicationFile(debug=debug)
        self.fileservice.local_file_path = 'test.ngc'
        self.fileservice.local_path = './ngc/'
        self.fileservice.remote_path = '/home/xy/'
        self.fileservice.remote_file_path = '/home/xy/test.ngc'
        self.fileservice.on_ready_changed.append(self._on_fileservice_ready)
        self.sd.register(self.fileservice)

        self.timer = None
        self.timer_interval = 0.1

        self.use_curses = use_curses
        if not self.use_curses:
            return

        self.messages = []

        self.screen = curses.initscr()
        self.screen.keypad(True)
        self.dro_window = curses.newwin(10, 40, 1, 2)
        self.status_window = curses.newwin(10, 40, 1, 44)
        self.command_window = curses.newwin(10, 40, 1, 86)
        self.connection_window = curses.newwin(10, 80, 12, 2)
        self.error_window = curses.newwin(20, 120, 12, 84)
        self.file_window = curses.newwin(10, 80, 1, 108)
        curses.noecho()
        curses.cbreak()

    def _on_status_synced(self, synced):
        if synced:
            self.timer = threading.Timer(self.timer_interval, self.status_timer_tick)
            self.timer.start()
        else:
            if self.timer:
                self.timer.cancel()
                self.timer = None

    def _on_fileservice_ready(self, ready):
        if ready:
            print('fileservice ready')
            self.fileservice.refresh_files()
            self.fileservice.wait_completed()
            print(self.fileservice.file_list)
            self.fileservice.remove_file('test.ngc')
            self.fileservice.wait_completed()

    def status_timer_tick(self):
        # if self.status.synced:
        # print('flood %s' % self.status.io.flood)
        if self.use_curses:
            self.update_screen()
        self.timer = threading.Timer(self.timer_interval, self.status_timer_tick)
        self.timer.start()

    def toggle_pin(self):
        self.halrcomp['coolant'] = not self.halrcomp['coolant']
        return True

    def update_screen(self):
        con = self.connection_window
        con.clear()
        con.border(0)
        con.addstr(1, 2, 'Connection')
        con.addstr(
            3, 4, 'Status: %s %s' % (str(self.status.synced), self.status.status_uri)
        )
        con.addstr(
            4,
            4,
            'Command: %s %s' % (str(self.command.connected), self.command.command_uri),
        )
        con.addstr(
            5, 4, 'Error: %s %s' % (str(self.error.connected), self.error.error_uri)
        )
        con.refresh()

        if not self.status.synced or not self.command.connected:
            return

        dro = self.dro_window
        dro.clear()
        dro.border(0)
        dro.addstr(1, 2, "DRO")
        for i, n in enumerate(['x', 'y', 'z']):  # range(self.status.config.axes):
            pos = str(getattr(self.status.motion.position, n))
            dro.addstr(3 + i, 4, '%s: %s' % (n, pos))
        dro.refresh()

        status = self.status_window
        status.clear()
        status.border(0)
        status.addstr(1, 2, 'Status')
        status.addstr(
            3,
            4,
            'Estop: %s'
            % str(self.status.task.task_state == application.EMC_TASK_STATE_ESTOP),
        )
        status.addstr(
            4,
            4,
            'Power: %s'
            % str(self.status.task.task_state == application.EMC_TASK_STATE_ON),
        )
        status.refresh()

        cmd = self.command_window
        cmd.clear()
        cmd.border(0)
        cmd.addstr(1, 2, 'Command')
        cmd.addstr(3, 4, 'Estop - F1')
        cmd.addstr(4, 4, 'Power - F2')
        cmd.refresh()

        error = self.error_window
        error.clear()
        error.border(0)
        error.addstr(1, 2, 'Notifications')
        self.messages += self.error.get_messages()
        pos = 0
        for message in self.messages:
            # msg_type = str(message['type'])
            for note in message['notes']:
                error.addstr(3 + pos, 4, str(note))
                pos += 1
        error.refresh()

        win = self.file_window
        win.clear()
        win.border(0)
        win.addstr(1, 2, 'File')
        win.addstr(3, 4, 'Status: %s' % self.fileservice.transfer_state)
        win.addstr(4, 4, 'Progress: %f' % self.fileservice.progress)
        win.refresh()

        self.screen.nodelay(True)
        c = self.screen.getch()
        if c == curses.KEY_F1:
            if self.status.task.task_state == application.EMC_TASK_STATE_ESTOP:
                ticket = self.command.set_task_state(
                    application.EMC_TASK_STATE_ESTOP_RESET
                )
                self.command.wait_completed(ticket=ticket, timeout=0.2)
            else:
                self.command.set_task_state(application.EMC_TASK_STATE_ESTOP)
                self.command.wait_completed(timeout=0.2)
        elif c == curses.KEY_F2:
            if self.status.task.task_state == application.EMC_TASK_STATE_ON:
                self.command.set_task_state(application.EMC_TASK_STATE_OFF)
            else:
                self.command.set_task_state(application.EMC_TASK_STATE_ON)
        elif c == curses.KEY_F3:
            self.fileservice.start_upload()

    def start(self):
        self.sd.start()

    def stop(self):
        self.sd.stop()

        if self.timer:
            self.timer.cancel()

        if self.use_curses:
            curses.endwin()


def main():
    mkconfig = config.Config()
    mkini = os.getenv("MACHINEKIT_INI")
    if mkini is None:
        mkini = mkconfig.MACHINEKIT_INI
    if not os.path.isfile(mkini):
        sys.stderr.write("MACHINEKIT_INI " + mkini + " does not exist\n")
        sys.exit(1)

    mki = configparser.ConfigParser()
    mki.read(mkini)
    uuid = mki.get("MACHINEKIT", "MKUUID")

    ui = TerminalUI(uuid=uuid, use_curses=True)
    ui.start()

    try:
        while True:
            time.sleep(0.5)
    except KeyboardInterrupt:
        pass

    print("stopping threads")
    ui.stop()

    # wait for all threads to terminate
    while threading.active_count() > 1:
        time.sleep(0.5)

    print("threads stopped")
    sys.exit(0)


if __name__ == "__main__":
    main()



# coding=utf-8
import threading

from machinetalk.protobuf.message_pb2 import Container
from .constants import (
    ENGAGE_BRAKE,
    RELEASE_BRAKE,
    JOG_STOP,
    JOG_CONTINUOUS,
    JOG_INCREMENT,
    SPINDLE_FORWARD,
    SPINDLE_REVERSE,
    SPINDLE_OFF,
    SPINDLE_INCREASE,
    SPINDLE_DECREASE,
    SPINDLE_CONSTANT,
)
from ..common import ComponentBase
from ..dns_sd import ServiceContainer, Service
from ..machinetalk_core.application.commandbase import CommandBase


class ApplicationCommand(ComponentBase, CommandBase, ServiceContainer):
    def __init__(self, debug=False):
        CommandBase.__init__(self, debuglevel=int(debug))
        ComponentBase.__init__(self)
        ServiceContainer.__init__(self)
        self.completed_condition = threading.Condition(threading.Lock())
        self.executed_condition = threading.Condition(threading.Lock())
        self.connected_condition = threading.Condition(threading.Lock())
        self.debug = debug

        # callbacks
        self.on_connected_changed = []

        self.connected = False

        self.ticket = 0  # stores the local ticket number
        self.executed_ticket = 0  # last tick number from executed feedback
        self.completed_ticket = 0  # last tick number from executed feedback
        self._executed_updated = False
        self._completed_updated = False

        # more efficient to reuse a protobuf message
        self._tx = Container()

        self._command_service = Service(type_='command')
        self.add_service(self._command_service)
        self.on_services_ready_changed.append(self._on_services_ready_changed)

    def _on_services_ready_changed(self, ready):
        self.command_uri = self._command_service.uri
        self.ready = ready

    def emccmd_executed_received(self, rx):
        with self.executed_condition:
            self.executed_ticket = rx.reply_ticket
            self._executed_updated = True
            self.executed_condition.notify()

    def emccmd_completed_received(self, rx):
        with self.completed_condition:
            self.completed_ticket = rx.reply_ticket
            self._completed_updated = True
            self.completed_condition.notify()

    def wait_executed(self, ticket=None, timeout=None):
        with self.executed_condition:
            if ticket is None:
                ticket = self.ticket
            if (
                ticket and ticket <= self.executed_ticket
            ):  # very likely that we already received the reply
                return True

            while True:
                self._executed_updated = False
                self.executed_condition.wait(timeout=timeout)
                if not self._executed_updated:
                    return False  # timeout
                if ticket == self.executed_ticket:
                    return True

    def wait_completed(self, ticket=None, timeout=None):
        with self.completed_condition:
            if ticket is None:
                ticket = self.ticket
            if (
                ticket and ticket < self.completed_ticket
            ):  # very likely that we already received the reply
                return True

            while True:
                self._completed_updated = False
                self.completed_condition.wait(timeout=timeout)
                if not self._completed_updated:
                    return False  # timeout
                if ticket == self.completed_ticket:
                    return True

    def wait_connected(self, timeout=None):
        with self.connected_condition:
            if self.connected:
                return True
            self.connected_condition.wait(timeout=timeout)
            return self.connected

    # slot
    def set_connected(self):
        self._update_connected(True)

    # slot
    def clear_connected(self):
        self._update_connected(False)

    def _update_connected(self, connected):
        with self.connected_condition:
            self.connected = connected
            self.connected_condition.notify()
        for cb in self.on_connected_changed:
            cb(connected)

    def _take_ticket(self):
        self.ticket += 1
        self._tx.ticket = self.ticket
        return self.ticket

    def abort(self, interpreter='execute'):
        if not self.connected:
            return None

        self._tx.interp_name = interpreter

        ticket = self._take_ticket()
        self.send_emc_task_abort(self._tx)
        return ticket

    def run_program(self, line_number, interpreter='execute'):
        if not self.connected:
            return None

        params = self._tx.emc_command_params
        params.line_number = line_number
        self._tx.interp_name = interpreter

        ticket = self._take_ticket()
        self.send_emc_task_plan_run(self._tx)
        return ticket

    def pause_program(self, interpreter='execute'):
        if not self.connected:
            return None

        self._tx.interp_name = interpreter

        ticket = self._take_ticket()
        self.send_emc_task_plan_pause(self._tx)
        return ticket

    def step_program(self, interpreter='execute'):
        if not self.connected:
            return None

        self._tx.interp_name = interpreter

        ticket = self._take_ticket()
        self.send_emc_task_plan_step(self._tx)
        return ticket

    def resume_program(self, interpreter='execute'):
        if not self.connected:
            return None

        self._tx.interp_name = interpreter

        ticket = self._take_ticket()
        self.send_emc_task_plan_resume(self._tx)
        return ticket

    def set_task_mode(self, mode, interpreter='execute'):
        if not self.connected:
            return

        params = self._tx.emc_command_params
        params.task_mode = mode
        self._tx.interp_name = interpreter

        ticket = self._take_ticket()
        self.send_emc_task_set_mode(self._tx)
        return ticket

    def set_task_state(self, state, interpreter='execute'):
        if not self.connected:
            return None

        params = self._tx.emc_command_params
        params.task_state = state
        self._tx.interp_name = interpreter

        ticket = self._take_ticket()
        self.send_emc_task_set_state(self._tx)
        return ticket

    def open_program(self, file_name, interpreter='execute'):
        if not self.connected:
            return None

        params = self._tx.emc_command_params
        params.path = file_name
        self._tx.interp_name = interpreter

        ticket = self._take_ticket()
        self.send_emc_task_plan_open(self._tx)
        return ticket

    def reset_program(self, interpreter='execute'):
        if not self.connected:
            return None

        self._tx.interp_name = interpreter

        ticket = self._take_ticket()
        self.send_emc_task_plan_init(self._tx)
        return ticket

    def execute_mdi(self, command, interpreter='execute'):
        if not self.connected:
            return None

        params = self._tx.emc_command_params
        params.command = command
        self._tx.interp_name = interpreter

        ticket = self._take_ticket()
        self.send_emc_task_plan_execute(self._tx)
        return ticket

    def set_spindle_brake(self, brake):
        if not self.connected:
            return None

        ticket = self._take_ticket()
        if brake == ENGAGE_BRAKE:
            self.send_emc_spindle_brake_engage(self._tx)
        elif brake == RELEASE_BRAKE:
            self.send_emc_spindle_brake_release(self._tx)
        return ticket

    def set_debug_level(self, debug_level):
        if not self.connected:
            return None

        params = self._tx.emc_command_params
        params.debug_level = debug_level
        self._tx.interp_name = debug_level

        ticket = self._take_ticket()
        self.send_emc_set_debug(self._tx)
        return ticket

    def set_feed_override(self, scale):
        if not self.connected:
            return None

        params = self._tx.emc_command_params
        params.scale = scale

        ticket = self._take_ticket()
        self.send_emc_traj_set_scale(self._tx)
        return ticket

    def set_flood_enabled(self, enable):
        if not self.connected:
            return None

        ticket = self._take_ticket()
        if enable:
            self.send_emc_coolant_flood_on(self._tx)
        else:
            self.send_emc_coolant_flood_off(self._tx)
        return ticket

    def home_axis(self, index):
        if not self.connected:
            return None

        params = self._tx.emc_command_params
        params.index = index

        ticket = self._take_ticket()
        self.send_emc_axis_home(self._tx)
        return ticket

    def jog(self, jog_type, axis, velocity=0.0, distance=0.0):
        if not self.connected:
            return None

        params = self._tx.emc_command_params
        params.index = axis

        if jog_type == JOG_STOP:
            ticket = self._take_ticket()
            self.send_emc_axis_abort(self._tx)
        elif jog_type == JOG_CONTINUOUS:
            params.velocity = velocity
            ticket = self._take_ticket()
            self.send_emc_axis_jog(self._tx)
        elif jog_type == JOG_INCREMENT:
            params.velocity = velocity
            params.distance = distance
            ticket = self._take_ticket()
            self.send_emc_axis_incr_jog(self._tx)
        else:
            self._tx.Clear()
            return None

        return ticket

    def load_tool_table(self):
        if not self.connected:
            return None

        ticket = self._take_ticket()
        self.send_emc_tool_load_tool_table(self._tx)
        return ticket

    def update_tool_table(self, tool_table):
        pass  # TODO

    def set_maximum_velocity(self, velocity):
        if not self.connected:
            return None

        params = self._tx.emc_command_params
        params.velocity = velocity

        ticket = self._take_ticket()
        self.send_emc_traj_set_max_velocity(self._tx)
        return ticket

    def set_mist_enabled(self, enable):
        if not self.connected:
            return None

        ticket = self._take_ticket()
        if enable:
            self.send_emc_coolant_mist_on(self._tx)
        else:
            self.send_emc_coolant_mist_off(self._tx)
        return ticket

    def override_limits(self):
        if not self.connected:
            return None

        ticket = self._take_ticket()
        self.send_emc_axis_override_limits(self._tx)
        return ticket

    def set_adaptive_feed_enabled(self, enable):
        if not self.connected:
            return None

        params = self._tx.emc_command_params
        params.enable = enable

        ticket = self._take_ticket()
        self.send_emc_motion_adaptive(self._tx)
        return ticket

    def set_analog_output(self, index, value):
        if not self.connected:
            return None

        params = self._tx.emc_command_params
        params.index = index
        params.value = value

        ticket = self._take_ticket()
        self.send_emc_motion_set_aout(self._tx)
        return ticket

    def set_block_delete_enabled(self, enable):
        if not self.connected:
            return None

        params = self._tx.emc_command_params
        params.enable = enable

        ticket = self._take_ticket()
        self.send_emc_task_plan_set_block_delete(self._tx)
        return ticket

    def set_digital_output(self, index, enable):
        if not self.connected:
            return None

        params = self._tx.emc_command_params
        params.index = index
        params.enable = enable

        ticket = self._take_ticket()
        self.send_emc_motion_set_dout(self._tx)
        return ticket

    def set_feed_hold_enabled(self, enable):
        if not self.connected:
            return None

        params = self._tx.emc_command_params
        params.enable = enable

        ticket = self._take_ticket()
        self.send_emc_traj_set_fh_enable(self._tx)
        return ticket

    def set_feed_override_enabled(self, enable):
        if not self.connected:
            return None

        params = self._tx.emc_command_params
        params.enable = enable

        ticket = self._take_ticket()
        self.send_emc_traj_set_fo_enable(self._tx)
        return ticket

    def set_axis_max_position_limit(self, axis, value):
        if not self.connected:
            return None

        params = self._tx.emc_command_params
        params.index = axis
        params.value = value

        ticket = self._take_ticket()
        self.send_emc_axis_set_max_position_limit(self._tx)
        return ticket

    def set_axis_min_position_limit(self, axis, value):
        if not self.connected:
            return None

        params = self._tx.emc_command_params
        params.index = axis
        params.value = value

        ticket = self._take_ticket()
        self.send_emc_axis_set_min_position_limit(self._tx)
        return ticket

    def set_optional_stop_enabled(self, enable):
        if not self.connected:
            return None

        params = self._tx.emc_command_params
        params.enable = enable

        ticket = self._take_ticket()
        self.send_emc_task_plan_set_optional_stop(self._tx)
        return ticket

    def set_spindle_override_enabled(self, enable):
        if not self.connected:
            return None

        params = self._tx.emc_command_params
        params.enable = enable

        ticket = self._take_ticket()
        self.send_emc_traj_set_so_enable(self._tx)
        return ticket

    def set_spindle(self, mode, velocity=0.0):
        if not self.connected:
            return None

        params = self._tx.emc_command_params
        if mode == SPINDLE_FORWARD:
            params.velocity = velocity
            ticket = self._take_ticket()
            self.send_emc_spindle_on(self._tx)
        elif mode == SPINDLE_REVERSE:
            params.velocity = velocity * -1.0
            ticket = self._take_ticket()
            self.send_emc_spindle_on(self._tx)
        elif mode == SPINDLE_OFF:
            ticket = self._take_ticket()
            self.send_emc_spindle_off(self._tx)
        elif mode == SPINDLE_INCREASE:
            ticket = self._take_ticket()
            self.send_emc_spindle_increase(self._tx)
        elif mode == SPINDLE_DECREASE:
            ticket = self._take_ticket()
            self.send_emc_spindle_decrease(self._tx)
        elif mode == SPINDLE_CONSTANT:
            ticket = self._take_ticket()
            self.send_emc_spindle_constant(self._tx)
        else:
            self._tx.Clear()
            return None

        return ticket

    def set_spindle_override(self, scale):
        if not self.connected:
            return None

        params = self._tx.emc_command_params
        params.scale = scale

        ticket = self._take_ticket()
        self.send_emc_traj_set_spindle_scale(self._tx)
        return ticket

    def set_teleop_enabled(self, enable):
        if not self.connected:
            return None

        params = self._tx.emc_command_params
        params.enable = enable

        ticket = self._take_ticket()
        self.send_emc_traj_set_teleop_enable(self._tx)
        return ticket

    def set_teleop_vector(self, a, b, c, u, v, w):
        if not self.connected:
            return None

        params = self._tx.emc_command_params
        pose = params.pose
        pose.a = a
        pose.b = b
        pose.c = c
        pose.u = u
        pose.v = v
        pose.w = w

        ticket = self._take_ticket()
        self.send_emc_traj_set_teleop_vector(self._tx)
        return ticket

    def set_tool_offset(
        self, index, zoffset, xoffset, diameter, frontangle, backangle, orientation
    ):
        if not self.connected:
            return None

        params = self._tx.emc_command_params
        tooldata = params.tool_data
        tooldata.index = index
        tooldata.zoffset = zoffset
        tooldata.xoffset = xoffset
        tooldata.diameter = diameter
        tooldata.frontangle = frontangle
        tooldata.backangle = backangle
        tooldata.orientation = orientation

        ticket = self._take_ticket()
        self.send_emc_tool_set_offset(self._tx)
        return ticket

    def set_trajectory_mode(self, mode):
        if not self.connected:
            return None

        params = self._tx.emc_command_params
        params.traj_mode = mode

        ticket = self._take_ticket()
        self.send_emc_traj_set_mode(self._tx)
        return ticket

    def unhome_axis(self, index):
        if not self.connected:
            return None

        params = self._tx.emc_command_params
        params.index = index

        ticket = self._take_ticket()
        self.send_emc_axis_unhome(self._tx)
        return ticket

    def shutdown(self):
        if not self.connected:
            return None

        ticket = self._take_ticket()
        self.send_shutdown(self._tx)
        return ticket



# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: ipcmsg.proto

import sys

_b = sys.version_info[0] < 3 and (lambda x: x) or (lambda x: x.encode("latin1"))
from google.protobuf.internal import enum_type_wrapper
from google.protobuf import descriptor as _descriptor
from google.protobuf import message as _message
from google.protobuf import reflection as _reflection
from google.protobuf import symbol_database as _symbol_database
from google.protobuf import descriptor_pb2

# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


DESCRIPTOR = _descriptor.FileDescriptor(
    name="ipcmsg.proto",
    package="pb",
    serialized_pb=_b(
        '\n\x0cipcmsg.proto\x12\x02pb"\xa2\x01\n\x07Message\x12\x1d\n\x04type\x18\x01 \x02(\x0e\x32\x0f.pb.MessageType\x12\x0c\n\x04\x61xis\x18\x02 \x01(\x05\x12\x10\n\x08velocity\x18\x03 \x01(\x01\x12\x10\n\x08\x64istance\x18\x04 \x01(\x01\x12\x1d\n\x08jog_type\x18\x05 \x01(\x0e\x32\x0b.pb.JogType\x12\t\n\x01x\x18\x06 \x01(\x01\x12\t\n\x01y\x18\x07 \x01(\x01\x12\x11\n\tconnected\x18\x08 \x01(\x08*?\n\x0bMessageType\x12\x0b\n\x07IPC_JOG\x10\x01\x12\x10\n\x0cIPC_POSITION\x10\x02\x12\x11\n\rIPC_CONNECTED\x10\x03*@\n\x07JogType\x12\x0c\n\x08JOG_STOP\x10\x00\x12\x12\n\x0eJOG_CONTINUOUS\x10\x01\x12\x13\n\x0fJOG_INCREMENTAL\x10\x02'
    ),
)
_sym_db.RegisterFileDescriptor(DESCRIPTOR)

_MESSAGETYPE = _descriptor.EnumDescriptor(
    name="MessageType",
    full_name="pb.MessageType",
    filename=None,
    file=DESCRIPTOR,
    values=[
        _descriptor.EnumValueDescriptor(
            name="IPC_JOG", index=0, number=1, options=None, type=None
        ),
        _descriptor.EnumValueDescriptor(
            name="IPC_POSITION", index=1, number=2, options=None, type=None
        ),
        _descriptor.EnumValueDescriptor(
            name="IPC_CONNECTED", index=2, number=3, options=None, type=None
        ),
    ],
    containing_type=None,
    options=None,
    serialized_start=185,
    serialized_end=248,
)
_sym_db.RegisterEnumDescriptor(_MESSAGETYPE)

MessageType = enum_type_wrapper.EnumTypeWrapper(_MESSAGETYPE)
_JOGTYPE = _descriptor.EnumDescriptor(
    name="JogType",
    full_name="pb.JogType",
    filename=None,
    file=DESCRIPTOR,
    values=[
        _descriptor.EnumValueDescriptor(
            name="JOG_STOP", index=0, number=0, options=None, type=None
        ),
        _descriptor.EnumValueDescriptor(
            name="JOG_CONTINUOUS", index=1, number=1, options=None, type=None
        ),
        _descriptor.EnumValueDescriptor(
            name="JOG_INCREMENTAL", index=2, number=2, options=None, type=None
        ),
    ],
    containing_type=None,
    options=None,
    serialized_start=250,
    serialized_end=314,
)
_sym_db.RegisterEnumDescriptor(_JOGTYPE)

JogType = enum_type_wrapper.EnumTypeWrapper(_JOGTYPE)
IPC_JOG = 1
IPC_POSITION = 2
IPC_CONNECTED = 3
JOG_STOP = 0
JOG_CONTINUOUS = 1
JOG_INCREMENTAL = 2


_MESSAGE = _descriptor.Descriptor(
    name="Message",
    full_name="pb.Message",
    filename=None,
    file=DESCRIPTOR,
    containing_type=None,
    fields=[
        _descriptor.FieldDescriptor(
            name="type",
            full_name="pb.Message.type",
            index=0,
            number=1,
            type=14,
            cpp_type=8,
            label=2,
            has_default_value=False,
            default_value=1,
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            options=None,
        ),
        _descriptor.FieldDescriptor(
            name="axis",
            full_name="pb.Message.axis",
            index=1,
            number=2,
            type=5,
            cpp_type=1,
            label=1,
            has_default_value=False,
            default_value=0,
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            options=None,
        ),
        _descriptor.FieldDescriptor(
            name="velocity",
            full_name="pb.Message.velocity",
            index=2,
            number=3,
            type=1,
            cpp_type=5,
            label=1,
            has_default_value=False,
            default_value=0,
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            options=None,
        ),
        _descriptor.FieldDescriptor(
            name="distance",
            full_name="pb.Message.distance",
            index=3,
            number=4,
            type=1,
            cpp_type=5,
            label=1,
            has_default_value=False,
            default_value=0,
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            options=None,
        ),
        _descriptor.FieldDescriptor(
            name="jog_type",
            full_name="pb.Message.jog_type",
            index=4,
            number=5,
            type=14,
            cpp_type=8,
            label=1,
            has_default_value=False,
            default_value=0,
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            options=None,
        ),
        _descriptor.FieldDescriptor(
            name="x",
            full_name="pb.Message.x",
            index=5,
            number=6,
            type=1,
            cpp_type=5,
            label=1,
            has_default_value=False,
            default_value=0,
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            options=None,
        ),
        _descriptor.FieldDescriptor(
            name="y",
            full_name="pb.Message.y",
            index=6,
            number=7,
            type=1,
            cpp_type=5,
            label=1,
            has_default_value=False,
            default_value=0,
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            options=None,
        ),
        _descriptor.FieldDescriptor(
            name="connected",
            full_name="pb.Message.connected",
            index=7,
            number=8,
            type=8,
            cpp_type=7,
            label=1,
            has_default_value=False,
            default_value=False,
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            options=None,
        ),
    ],
    extensions=[],
    nested_types=[],
    enum_types=[],
    options=None,
    is_extendable=False,
    extension_ranges=[],
    oneofs=[],
    serialized_start=21,
    serialized_end=183,
)

_MESSAGE.fields_by_name["type"].enum_type = _MESSAGETYPE
_MESSAGE.fields_by_name["jog_type"].enum_type = _JOGTYPE
DESCRIPTOR.message_types_by_name["Message"] = _MESSAGE
DESCRIPTOR.enum_types_by_name["MessageType"] = _MESSAGETYPE
DESCRIPTOR.enum_types_by_name["JogType"] = _JOGTYPE

Message = _reflection.GeneratedProtocolMessageType(
    "Message",
    (_message.Message,),
    dict(
        DESCRIPTOR=_MESSAGE,
        __module__="ipcmsg_pb2"
        # @@protoc_insertion_point(class_scope:pb.Message)
    ),
)
_sym_db.RegisterMessage(Message)


# @@protoc_insertion_point(module_scope)



#!/usr/bin/env python
import zmq
import time
import sys
import threading
import argparse
from functools import partial
from pymachinetalk.dns_sd import Service, ServiceDiscovery
from machinetalk.protobuf.message_pb2 import Container
from machinetalk.protobuf.types_pb2 import *


class TestClass(object):
    def __init__(self, debug=False):
        self.debug = debug
        context = zmq.Context()
        context.linger = 0
        self.context = context

        self.sd = ServiceDiscovery()
        services = ['launchercmd', 'halrcmd', 'command', 'config']
        for name in services:
            service = Service(type_=name)
            self.sd.register(service)
            service.on_ready_changed.append(
                partial(self.service_ready, service=service)
            )

        self._tx = Container()
        self._rx = Container()

    def start(self):
        self.sd.start()

    def stop(self):
        self.sd.stop()

    def service_ready(self, ready, service):
        if ready:
            self.send_broken(service.name, service.uri)
            self.send_broker_msg(service.name, service.uri)

    def send_broken(self, name, dsn):
        socket = self.context.socket(zmq.DEALER)
        socket.setsockopt(zmq.LINGER, 0)
        socket.setsockopt(zmq.RCVTIMEO, 1000)
        if self.debug:
            print('connecting to %s' % dsn)
        socket.connect(dsn)
        bogus = 'random_string'
        if self.debug:
            print('sending bogus string %s' % bogus)
        socket.send(bogus)
        answer = ''
        try:
            answer = socket.recv()
        except zmq.error.Again as e:
            print('test failed, answer timed out %s' % name)
            return
        self._rx.ParseFromString(answer)
        if self.debug:
            print('received answer %s' % self._rx)
        if self._rx.type == MT_ERROR:
            print('test successfull, %s handles broken messages' % name)
        else:
            print('test failed, %s does not handle broken messages' % name)

    def send_broker_msg(self, name, dsn):
        frontend = self.context.socket(zmq.ROUTER)
        backend = self.context.socket(zmq.DEALER)
        backend.setsockopt(zmq.IDENTITY, 'broker')
        socket = self.context.socket(zmq.DEALER)
        socket.setsockopt(zmq.IDENTITY, 'client')
        if self.debug:
            print('connecting backend to %s' % dsn)
        backend.connect(dsn)
        ipc = 'ipc://broker'
        if self.debug:
            print('connecting frontend to %s' % ipc)
        frontend.bind(ipc)
        if self.debug:
            print('connecting socket to %s' % ipc)
        socket.connect(ipc)
        if self.debug:
            print('sending ping via socket')
        self._tx.Clear()
        self._tx.type = MT_PING
        socket.send(self._tx.SerializeToString())
        if self.debug:
            print('receiving with broker:')
        recv = frontend.recv_multipart()
        if self.debug:
            print(recv)
            print('forwarding to host')
        backend.send_multipart(recv)
        if self.debug:
            print('receiving with backend:')
        recv = backend.recv_multipart()
        if self.debug:
            print(recv)
            print('sending data to frontend:')
        frontend.send_multipart(recv)
        if self.debug:
            print('receiving data with socket:')
        recv = socket.recv()
        self._rx.ParseFromString(recv)
        if self.debug:
            print(self._rx)
        if self._rx.type == MT_PING_ACKNOWLEDGE:
            print('test successfull, %s handles multiple ids' % name)
        else:
            print('test failed, %s does not correctly handle multiple ids' % name)


def main():
    parser = argparse.ArgumentParser(
        description='tests Machinetalk RPC sockets behavior on broken messages and multiple ids'
    )
    parser.add_argument('-d', '--debug', help='Enable debug mode', action='store_true')
    args = parser.parse_args()

    test_class = TestClass(debug=args.debug)
    test_class.start()

    try:
        while True:
            time.sleep(0.5)
    except KeyboardInterrupt:
        pass

    test_class.stop()

    print("stopping threads")
    # testClass.stop()

    # wait for all threads to terminate
    while threading.active_count() > 1:
        time.sleep(0.1)

    print("threads stopped")
    sys.exit(0)


if __name__ == "__main__":
    main()



# coding=utf-8
import threading

from ..common import ComponentBase
from ..dns_sd import ServiceContainer, Service
from ..machinetalk_core.application.errorbase import ErrorBase


class ApplicationError(ComponentBase, ErrorBase, ServiceContainer):
    def __init__(self, debug=False):
        ErrorBase.__init__(self, debuglevel=int(debug))
        ComponentBase.__init__(self)
        ServiceContainer.__init__(self)
        self.message_lock = threading.Lock()
        self.connected_condition = threading.Condition(threading.Lock())
        self.debug = debug

        # callbacks
        self.on_connected_changed = []

        self.connected = False
        self.channels = {'error', 'text', 'display'}
        self.error_list = []

        self._error_service = Service(type_='error')
        self.add_service(self._error_service)
        self.on_services_ready_changed.append(self._on_services_ready_changed)

    def _on_services_ready_changed(self, ready):
        self.error_uri = self._error_service.uri
        self.ready = ready

    def wait_connected(self, timeout=None):
        with self.connected_condition:
            if self.connected:
                return True
            self.connected_condition.wait(timeout=timeout)
            return self.connected

    def emc_nml_error_received(self, _, rx):
        self._error_message_received(rx)

    def emc_nml_text_received(self, _, rx):
        self._error_message_received(rx)

    def emc_nml_display_received(self, _, rx):
        self._error_message_received(rx)

    def emc_operator_text_received(self, _, rx):
        self._error_message_received(rx)

    def emc_operator_display_received(self, _, rx):
        self._error_message_received(rx)

    def emc_operator_error_received(self, _, rx):
        self._error_message_received(rx)

    def _error_message_received(self, rx):
        error = {'type': rx.type, 'notes': []}
        with self.message_lock:
            for note in rx.note:
                error['notes'].append(note)
                self.error_list.append(error)

    # slot
    def update_topics(self):
        self.clear_error_topics()
        for channel in self.channels:
            self.add_error_topic(channel)

    # slot
    def set_connected(self):
        self._update_connected(True)

    # slot
    def clear_connected(self):
        self._update_connected(False)

    def _update_connected(self, connected):
        with self.connected_condition:
            self.connected = connected
            self.connected_condition.notify()
        for cb in self.on_connected_changed:
            cb(connected)

    # returns all received messages and clears the buffer
    def get_messages(self):
        with self.message_lock:
            messages = list(self.error_list)  # make sure to return a copy
            self.error_list = []
            return messages



# coding=utf-8
import sys


class MessageObject(object):
    def __init__(self):
        self.is_position = False
        self.id_map = {}

    def __str__(self):
        output = ''
        for attr in dir(self)[3:]:
            output += '%s: %s\n' % (attr, getattr(self, attr))
        return output

    def __getitem__(self, index):
        if self.is_position:
            mapping = ['x', 'y', 'z', 'a', 'b', 'c', 'u', 'v', 'w']
            return getattr(self, mapping[index])
        else:
            raise RuntimeError("Object does not support indexed access")


def recurse_descriptor(descriptor, obj):
    for field in descriptor.fields:
        value = None

        if field.type == field.TYPE_BOOL:
            value = False
        elif field.type == field.TYPE_DOUBLE or field.type == field.TYPE_FLOAT:
            value = 0.0
        elif (
            field.type == field.TYPE_INT32
            or field.type == field.TYPE_INT64
            or field.type == field.TYPE_UINT32
            or field.type == field.TYPE_UINT64
        ):
            value = 0
        elif field.type == field.TYPE_STRING:
            value = ''
        elif field.type == field.TYPE_ENUM:
            value = 0
        elif field.type == field.TYPE_MESSAGE:
            value = MessageObject()
            msg_descriptor = field.message_type
            if msg_descriptor.name == 'Position':
                value.is_position = True
            recurse_descriptor(msg_descriptor, value)

        if field.label == field.LABEL_REPEATED:
            delattr(value, 'index')
            attributes = dir(value)
            if len(attributes) == 4:  # only single attribute
                value = getattr(value, attributes[-1])
            value = [value]

        setattr(obj, field.name, value)
        obj.id_map[field.number] = field.name


def recurse_message(message, obj, field_filter=''):
    for descriptor in message.DESCRIPTOR.fields:
        filter_enabled = field_filter != ''
        # TODO: handle special file case here...

        if descriptor.number in obj.id_map:
            name = obj.id_map[descriptor.number]
        else:
            continue  # we do not know the object

        if filter_enabled and name != field_filter:
            continue

        if descriptor.label != descriptor.LABEL_REPEATED:
            if message.HasField(name):
                if descriptor.type == descriptor.TYPE_MESSAGE:
                    sub_obj = getattr(obj, name)
                    recurse_message(getattr(message, name), sub_obj)
                else:
                    setattr(obj, name, getattr(message, name))
        else:
            if descriptor.type == descriptor.TYPE_MESSAGE:
                array = getattr(obj, name)
                repeated = getattr(message, name)
                for sub_message in repeated:
                    index = sub_message.index

                    while len(array) < (index + 1):
                        array.append(MessageObject())

                    if len(sub_message.DESCRIPTOR.fields) == 2:
                        sub_obj = MessageObject()
                        recurse_descriptor(sub_message.DESCRIPTOR, sub_obj)
                        recurse_message(sub_message, sub_obj)
                        delattr(sub_obj, 'index')
                        value = getattr(sub_obj, dir(sub_obj)[-1])
                    else:
                        sub_obj = array[index]
                        recurse_message(sub_message, sub_obj)
                        value = sub_obj
                    array[index] = value


# noinspection PyUnresolvedReferences
class ComponentBase(object):
    def __init__(self):
        self._ready = False
        self.on_ready_changed = []
        self.on_error_string_changed.append(self._on_error_string_changed)

    @property
    def ready(self):
        return self._ready

    @ready.setter
    def ready(self, ready):
        if ready is self._ready:
            return

        self._ready = ready
        if ready:
            self.start()
        else:
            self.stop()

        for cb in self.on_ready_changed:
            cb(ready)

    def _on_error_string_changed(self, string):
        sys.stderr.write('Error: %s\n' % string)



# coding=utf-8
import pytest
import sys


@pytest.fixture
def halremote():
    from pymachinetalk import halremote

    return halremote


@pytest.fixture
def dns_sd():
    from pymachinetalk import dns_sd

    return dns_sd


@pytest.mark.skipif(
    sys.version_info >= (3, 0),
    reason="Integration tests hang for some reason with Python3",
)
def test_halremote_integration(halremote, dns_sd):
    rcomp = halremote.RemoteComponent('test')
    rcomp.newpin('foo', halremote.HAL_BIT, halremote.HAL_OUT)
    rcomp.newpin('bar', halremote.HAL_FLOAT, halremote.HAL_IN)

    sd = dns_sd.ServiceDiscovery()
    sd.register(rcomp)



# coding=utf-8
import socket

import pytest


@pytest.fixture
def dns_sd():
    from pymachinetalk import dns_sd

    return dns_sd


@pytest.fixture
def sd():
    from pymachinetalk import dns_sd

    sd = dns_sd.ServiceDiscovery()
    return sd


def test_registering_services_from_service_container_works(dns_sd, sd):
    service = dns_sd.Service()
    discoverable = dns_sd.ServiceContainer()
    discoverable.services.append(service)

    sd.register(discoverable)

    assert service in sd.services


def test_registering_service_directly_works(dns_sd, sd):
    service = dns_sd.Service()

    sd.register(service)

    assert service in sd.services


def test_registering_anything_else_fails(sd):
    item = object()

    try:
        sd.register(item)
    except TypeError:
        assert True

    assert item not in sd.services


def test_registering_when_running_throws_error(dns_sd, sd):
    service = dns_sd.Service()

    def dummy():
        pass

    sd._start_discovery = dummy
    sd.start()

    try:
        sd.register(service)
    except RuntimeError:
        assert True

    assert service not in sd.services


def test_unregistering_service_directly_works(dns_sd, sd):
    service = dns_sd.Service()
    sd.register(service)

    sd.unregister(service)

    assert service not in sd.services


def test_unregistering_services_from_service_container_works(dns_sd, sd):
    service = dns_sd.Service()
    discoverable = dns_sd.ServiceContainer()
    discoverable.services.append(service)
    sd.register(discoverable)

    sd.unregister(discoverable)

    assert service not in sd.services


def test_unregistering_anything_else_fails(sd):
    item = 34

    try:
        sd.unregister(item)
    except TypeError:
        assert True

    assert item not in sd.services


def test_unregistering_when_running_throws_error(dns_sd, sd):
    service = dns_sd.Service()

    def dummy():
        pass

    sd._start_discovery = dummy
    sd.start()

    try:
        sd.unregister(service)
    except RuntimeError:
        assert True

    assert service not in sd.services


class ServiceInfoFactory(object):
    @staticmethod
    def create(
        base_type='machinekit',
        domain='local',
        sd_protocol='tcp',
        name='Hugo on Franz',
        service=b'halrcomp',
        uuid=b'12345678',
        host='127.0.0.1',
        protocol='tcp',
        port=12345,
        version=0,
        properties=None,
        server='127.0.0.1',
        address=None,
    ):
        from zeroconf import ServiceInfo

        typestring = '_%s._%s.%s.' % (base_type, sd_protocol, domain)
        dsn = b'%s://%s:%i' % (protocol.encode(), host.encode(), port)
        if properties is None:
            properties = {
                b'uuid': uuid,
                b'service': service,
                b'dsn': dsn,
                b'version': version,
            }
        return ServiceInfo(
            type_=typestring,
            name='%s %s.%s' % (name, host, typestring),
            properties=properties,
            address=socket.inet_aton(address or host),
            port=port,
            server=server,
        )


@pytest.fixture
def zeroconf(mocker):
    from zeroconf import Zeroconf

    service_info = ServiceInfoFactory().create()
    zeroconf_stub = mocker.stub(name='get_service_info')
    zeroconf_stub.return_value = service_info
    stub_object = Zeroconf()
    stub_object.get_service_info = zeroconf_stub
    return stub_object


@pytest.fixture
def zeroconf_without_service_info(mocker):
    from zeroconf import Zeroconf

    zeroconf_stub = mocker.stub(name='get_service_info')
    zeroconf_stub.return_value = None
    stub_object = Zeroconf()
    stub_object.get_service_info = zeroconf_stub
    return stub_object


def test_service_discovered_updates_registered_services(dns_sd, sd, zeroconf):
    service = dns_sd.Service(type_='halrcomp')
    sd.register(service)

    sd.add_service(
        zeroconf,
        '_machinekit._tcp.local.',
        'Foo on Bar 127.0.0.1._machinekit._tcp.local.',
    )

    assert service.ready is True


def test_service_disappeared_updates_registered_services(dns_sd, sd, zeroconf):
    service = dns_sd.Service(type_='halrcomp')
    sd.register(service)

    zeroconf.get_service_info.return_value = ServiceInfoFactory.create(
        name='Foo on Bar', host='127.0.0.1'
    )
    sd.add_service(
        zeroconf,
        '_machinekit._tcp.local.',
        'Foo on Bar 127.0.0.1._machinekit._tcp.local.',
    )
    zeroconf.get_service_info.return_value = None
    sd.remove_service(
        zeroconf,
        '_machinekit._tcp.local.',
        'Foo on Bar 127.0.0.1._machinekit._tcp.local.',
    )

    assert service.ready is False


def test_stopping_service_discovery_resets_all_services(dns_sd, sd, zeroconf):
    service1 = dns_sd.Service(type_='halrcomp')
    sd.register(service1)
    service2 = dns_sd.Service(type_='halrcmd')
    sd.register(service2)
    sd.browser = object()  # dummy
    sd.add_service(
        zeroconf,
        '_machinekit._tcp.local.',
        'Foo on Bar 127.0.0.1._machinekit._tcp.local.',
    )

    sd.stop()

    assert service1.ready is False
    assert service2.ready is False


def test_service_discovered_without_service_info_does_not_update_registered_services(
    dns_sd, sd, zeroconf_without_service_info
):
    service = dns_sd.Service(type_='halrcomp')
    sd.register(service)

    sd.add_service(
        zeroconf_without_service_info,
        '_machinekit._tcp.local.',
        'Foo on Bar 127.0.0.1._machinekit._tcp.local.',
    )

    assert service.ready is False


def test_service_info_sets_all_relevant_values_of_service(dns_sd):
    service = dns_sd.Service(type_='halrcomp')
    service_info = ServiceInfoFactory().create(
        name='Foo on Bar',
        uuid=b'987654321',
        version=5,
        host='10.0.0.10',
        protocol='tcp',
        port=12456,
        server='sandybox.local',
    )

    service.add_service_info(service_info)

    assert service.uri == 'tcp://10.0.0.10:12456'
    assert service.name == service_info.name
    assert service.uuid == '987654321'
    assert service.version == 5
    assert service.host_name == 'sandybox.local'
    assert service.host_address == '10.0.0.10'


def test_service_info_resolves_local_hostname_if_matched(dns_sd):
    service = dns_sd.Service(type_='halrcomp')
    service_info = ServiceInfoFactory().create(
        host='sandybox.local',
        protocol='tcp',
        port=12456,
        server='sandybox.local',
        address='10.0.0.10',
    )

    service.add_service_info(service_info)

    assert service.uri == 'tcp://10.0.0.10:12456'


def test_service_info_returs_raw_uri_if_hostname_is_not_matched(dns_sd):
    service = dns_sd.Service(type_='halrcomp')
    service_info = ServiceInfoFactory().create(
        host='thinkpad.local',
        protocol='tcp',
        port=12456,
        server='sandybox.local',
        address='10.0.0.10',
    )

    service.add_service_info(service_info)

    assert service.uri == 'tcp://thinkpad.local:12456'


def test_service_info_with_incomplete_values_is_ignored_by_service(dns_sd):
    service = dns_sd.Service(type_='launcher')
    service_info = ServiceInfoFactory().create(properties={})

    service.add_service_info(service_info)

    assert service.uri == ''
    assert service.uuid == ''
    assert service.version == b''


def test_removing_service_info_resets_all_relevant_values_of_service(dns_sd):
    service = dns_sd.Service(type_='blahus')
    service_info = ServiceInfoFactory().create()
    service.add_service_info(service_info)

    service.remove_service_info(service_info.name)

    assert service.uri == ''
    assert service.name == ''
    assert service.uuid == ''
    assert service.version == 0
    assert service.host_name == ''
    assert service.host_address == ''


def test_clearing_service_infos_resets_values_of_service(dns_sd):
    service = dns_sd.Service(type_='foobar')
    service.add_service_info(ServiceInfoFactory().create())
    service.add_service_info(ServiceInfoFactory().create())

    service.clear_service_infos()

    assert service.ready is False
    assert service.uri == ''


def test_setting_ready_property_of_service_triggers_callback(dns_sd):
    cb_called = [False]

    def cb(_):
        cb_called[0] = True

    service = dns_sd.Service(type_='halrcomp')
    service.on_ready_changed.append(cb)
    service_info = ServiceInfoFactory().create()

    service.add_service_info(service_info)

    assert cb_called[0] is True


def test_discoverable_adding_service_works(dns_sd):
    discoverable = dns_sd.ServiceContainer()
    service = dns_sd.Service(type_='foo')

    discoverable.add_service(service)

    assert service in discoverable.services


def test_discoverable_adding_anything_else_fails(dns_sd):
    discoverable = dns_sd.ServiceContainer()
    item = object()

    try:
        discoverable.add_service(item)
        assert False
    except TypeError:
        assert True

    assert item not in discoverable.services


def test_discoverable_removing_service_works(dns_sd):
    discoverable = dns_sd.ServiceContainer()
    service = dns_sd.Service(type_='foo')

    discoverable.add_service(service)
    discoverable.remove_service(service)

    assert service not in discoverable.services


def test_discoverable_remvoing_anything_else_fails(dns_sd):
    discoverable = dns_sd.ServiceContainer()
    item = object()

    try:
        discoverable.remove_service(item)
        assert False
    except TypeError:
        assert True

    assert item not in discoverable.services


def test_discoverable_all_services_ready_set_services_ready(dns_sd):
    discoverable = dns_sd.ServiceContainer()
    service1 = dns_sd.Service(type_='foo')
    discoverable.add_service(service1)
    service2 = dns_sd.Service(type_='bar')
    discoverable.add_service(service2)

    service1.ready = True
    service2.ready = True

    assert discoverable.services_ready is True


def test_discoverable_not_all_services_ready_unsets_services_ready(dns_sd):
    discoverable = dns_sd.ServiceContainer()
    service1 = dns_sd.Service(type_='foo')
    discoverable.add_service(service1)
    service2 = dns_sd.Service(type_='bar')
    discoverable.add_service(service2)

    service1.ready = True
    service2.ready = True
    service1.ready = False

    assert discoverable.services_ready is False


def test_discoverable_services_ready_changed_calls_callback(dns_sd):
    cb_called = [False]

    def cb(_):
        cb_called[0] = True

    discoverable = dns_sd.ServiceContainer()
    discoverable.on_services_ready_changed.append(cb)

    discoverable.services_ready = True

    assert cb_called[0] is True


def test_service_discovery_filter_accept_correct_uuid(dns_sd):
    service_info = ServiceInfoFactory().create(uuid=b'987654321')
    filter_ = dns_sd.ServiceDiscoveryFilter(txt_records={b'uuid': b'987654321'})

    assert filter_.matches_service_info(service_info) is True


def test_service_discovery_filter_reject_wrong_uuid(dns_sd):
    service_info = ServiceInfoFactory().create(uuid=b'123456789')
    filter_ = dns_sd.ServiceDiscoveryFilter(txt_records={b'uuid': b'987654321'})

    assert filter_.matches_service_info(service_info) is False


def test_service_discovery_filter_accept_fuzzy_name(dns_sd):
    service_info = ServiceInfoFactory().create(name='Hello World')
    filter_ = dns_sd.ServiceDiscoveryFilter(name='Hello')

    assert filter_.matches_service_info(service_info) is True
    assert filter_.matches_name(service_info.name) is True


def test_service_discovery_filter_accept_exact_matching_name(dns_sd):
    service_info = ServiceInfoFactory().create(name='Foo')
    filter_ = dns_sd.ServiceDiscoveryFilter(name='Foo')

    assert filter_.matches_service_info(service_info) is True
    assert filter_.matches_name(service_info.name) is True


def test_service_discovery_filter_reject_non_matching_name(dns_sd):
    service_info = ServiceInfoFactory().create(name='Carolus Rex')
    filter_ = dns_sd.ServiceDiscoveryFilter(name='Adolfus Maximus')

    assert filter_.matches_service_info(service_info) is False
    assert filter_.matches_name(service_info.name) is False


def test_service_discovery_filter_passing_wrong_object_fails(dns_sd):
    filter_ = dns_sd.ServiceDiscoveryFilter()

    try:
        filter_.matches_service_info(object())
        assert False
    except TypeError:
        assert True


def test_service_discovery_filters_out_discovered_service_with_wrong_uuid(
    dns_sd, sd, zeroconf
):
    service = dns_sd.Service(type_='halrcomp')
    sd.register(service)
    sd.filter = dns_sd.ServiceDiscoveryFilter(txt_records={b'uuid': b'87654321'})

    sd.add_service(
        zeroconf,
        '_machinekit._tcp.local.',
        'Machinekit on MyBox 12.0.0.1._machinekit._tcp.local.',
    )

    assert service.ready is False


def test_service_discovery_filters_in_discovered_service_with_correct_uuid(
    dns_sd, sd, zeroconf
):
    service = dns_sd.Service(type_='halrcomp')
    sd.register(service)
    sd.filter = dns_sd.ServiceDiscoveryFilter(txt_records={b'uuid': b'12345678'})

    sd.add_service(
        zeroconf,
        '_machinekit._tcp.local.',
        'SuperPrint 192.168.7.2._machinekit._tcp.local.',
    )
    assert service.ready is True


def test_service_discovery_filters_in_disappeared_service_with_correct_uuid(
    dns_sd, sd, zeroconf
):
    service = dns_sd.Service(type_='halrcomp')
    sd.register(service)
    sd.filter = dns_sd.ServiceDiscoveryFilter(txt_records={b'uuid': b'12345678'})

    zeroconf.get_service_info.return_value = ServiceInfoFactory.create(
        name='SuperPrint', host='192.168.7.2'
    )
    sd.add_service(
        zeroconf,
        '_machinekit._tcp.local.',
        'SuperPrint 192.168.7.2._machinekit._tcp.local.',
    )
    sd.remove_service(
        zeroconf,
        '_machinekit._tcp.local.',
        'SuperPrint 192.168.7.2._machinekit._tcp.local.',
    )
    assert service.ready is False
